#!/usr/bin/perl

=pod 

=head1 NAME

tp_blastp.pl 

=head1  SYNOPSIS 

tp_blastp.pl -i <FASTA_FILE> -o <BLAST_REPORT> [-e <float>] [-d <file>]
[--help]

=head1 DESCRIPTION

    	Blastp against Swiss-Prot database

=head1 AUTHORS

    	Sebastien.Carrere@toulouse.inra.fr

=cut

BEGIN
{
		use FindBin;
        use lib "$FindBin::Bin/../../playmoby/lib";
}

use strict;
use ParamParser;
use Appli;
use Cwd 'abs_path';



my $BLAST_BIN = $FindBin::Bin . '/../ext/blast_bin/blastall -p blastp -a 1 -m 8 ';
my $VERSION = `$FindBin::Bin/../ext/blast_bin/blastall - | grep ^blastall | cut -d ' ' -f2`;
chomp $VERSION;

my $DB_DIR     = '/db/generic';
my %H_DBS      = ('sp' => 'Swiss-Prot database', 'enzyme' => 'Enzyme database');
my $DEFAULT_DB = 'enzyme';
my %H_DEFAULT_VALUES = (e => 0.001, v => 20);

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG', ("o=s", "d=s", "i=s", "e=f", "v=i", "mobyle", "pmbtest", "debug"));
    
    &SetAppli(\$o_param);

    $o_param->AssertFileExists('i');
    $o_param->AssertDefined('o'); 
    $o_param->SetUnlessDefined('d', $DEFAULT_DB);
    $o_param->SetUnlessDefined('e', $H_DEFAULT_VALUES{e});
    $o_param->SetUnlessDefined('v', $H_DEFAULT_VALUES{v});
    
	$o_param->AssertAllowedValue('d', keys %H_DBS);
    
    my $blast_cmd =
      $BLAST_BIN 
      . ' -d ' . $DB_DIR . '/' . $o_param->Get('d') 
      . ' -i ' . $o_param->Get('i') 
      . ' -o ' . $o_param->Get('o') 
      . ' -e ' . $o_param->Get('e')
      . ' -v ' . $o_param->Get('v');
    print STDERR "INFO - $blast_cmd\n" if ($o_param->IsDefined('debug'));
    my $ret = system ($blast_cmd);
    print STDERR "ERROR -  $! \n" if ($ret < 0);
    
    
}

sub SetAppli
{
	my $ro_param = shift;
    my %h_general = (
                     name     => 'BlastP',
                     descr    => 'Blastp against Swiss-Prot or Enzyme',
                     category => 'NCBI_Blast',
                     authors  => 'Sebastien.Carrere@toulouse.inra.fr',
                     version  => $VERSION,
                     cmd      => abs_path($0)
                     );

    my %h_inputs =
      ('Query' => {descr => 'Query protein', type_biomoby => 'FASTA_AA',  cmd => '-i $value'});
    my %h_outputs =
      ('BlastReport' =>
        {descr => 'NCBI blast report', type_biomoby => 'text-formatted', cmd => '-o $value'});

    my %h_parameters = (
                        'expect_value' => {
                                           descr  => 'Expectation value',
                                           type_biomoby => 'Float',
                                           min 	   => 0.000000000001,
                                           max     => 10,
                                           default      => $H_DEFAULT_VALUES{e},
                                           cmd          => '-e $value'
                          },
                          'descriptions' => {
                                           descr  => 'Number of description',
                                           type_biomoby => 'Integer',
                                           min => 1,
                                           max => 500,
                                           default      => $H_DEFAULT_VALUES{v},
                                           cmd          => '-v $value'
                          },
                          'database' => {
                                       descr  => 'Target database',
                                       type_biomoby => 'String',
                                       default      => $DEFAULT_DB,
                                       enum         => \%H_DBS,
                                       cmd          => '-d $value'
                          }
                          );

    my $o_appli = New Appli(-general => \%h_general);
    $o_appli->SetInputs(%h_inputs);
    $o_appli->SetOutputs(%h_outputs);
    $o_appli->SetParams(%h_parameters);
    $$ro_param->SetAppli($o_appli); 
    
    
    exit print STDOUT $o_appli->GetMobyleXml()       if ($$ro_param->IsDefined('mobyle'));
    exit print STDOUT $o_appli->GetPlaymobyTestXml() if ($$ro_param->IsDefined('pmbtest'));
    return ($o_appli);

}


